#!/bin/sh
#-------------------------------------------------------------------------------
# /etc/config.d/apache2.sh - configuration generator script for Apache
#
# Creation:     2004-01-01 fw
# Last Update:  $Id: apache2.sh 106294 2023-09-19 12:34:47Z schlotze $
#
# Copyright (c) 2004-2023 the eisfair team, team(at)eisfair(dot)org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#-------------------------------------------------------------------------------

#echo "Executing $0 ..."
#exec 2> /tmp/apache2-trace$$.log
#set -x

. /var/install/include/eislib

quietmode=false
quiet=''

while [ ${#} -gt 0 ]
do
    case ${1} in
    --quiet)
        quietmode=true
        quiet='--quiet'
        shift
        ;;
    *)
        shift
        ;;
    esac
done

# ---------------------------------------------------------------------------
# my echo
# ---------------------------------------------------------------------------
myecho()
{
    if ! ${quietmode:-false}
    then
        mecho ${1} "${2}"
    fi
}

#========================================================================================
# check_for_autostart
#========================================================================================
check_for_autostart ()
{
    if [ "$START_APACHE2" = "yes" ]
    then
        if /usr/sbin/service is-enabled apache2.service
        then
            /usr/sbin/service daemon-reload
            /usr/sbin/service restart apache2-htcacheclean.service
        else
           /usr/sbin/service enable apache2.service
           /usr/sbin/service enable apache2-htcacheclean.service
           /usr/sbin/service start apache2-htcacheclean.service
        fi
    else
        /usr/sbin/service stop apache2.service
        /usr/sbin/service stop apache2-htcacheclean.service
        /usr/sbin/service disable apache2.service
        /usr/sbin/service disable apache2-htcacheclean.service
    fi
}

myecho --std "Creating Apache2 configuration ..."

pgmname=$0
conffile=/etc/config.d/apache2
packages_name="apache2"

chmod 600 $conffile

. $conffile
. /var/install/include/apache2lib
. /etc/apache2/APACHE2    # include definitions of user/group

# check for new installation
if [ -f "/etc/apache2/new-install" ]
then
    _new_install=true
    rm  -f "/etc/apache2/new-install"
fi

# prepare apache directories
prepare_apache_directories

# prepare doc-root
if [ ! -d ${APACHE2_DOCUMENT_ROOT} ]
then
    mkdir -p ${APACHE2_DOCUMENT_ROOT}
    cp /usr/share/apache2/www/htdocs/* ${APACHE2_DOCUMENT_ROOT}
    chown -R ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} ${APACHE2_DOCUMENT_ROOT}
fi
if [ ! -d ${APACHE2_SCRIPT_DIR} ]
then
    mkdir -p ${APACHE2_SCRIPT_DIR}
    cp /usr/share/apache2/www/cgi-bin/* ${APACHE2_SCRIPT_DIR}
    chown -R ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} ${APACHE2_SCRIPT_DIR}
fi
tmp_error_log=`dirname ${APACHE2_ERROR_LOG}`
if [ ! -d $tmp_error_log ]
then
    mkdir -p $tmp_error_log
    chown -R ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} ${tmp_error_log}
fi
tmp_access_log=`dirname ${APACHE2_ACCESS_LOG}`
if [ ! -d $tmp_access_log ]
then
    mkdir -p $tmp_access_log
    chown -R ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} ${tmp_access_log}
fi

vhost_alias_path='/etc/apache2/vhost'

# add trailing slash if necessary
APACHE2_SSL_LOGDIR=`echo ${APACHE2_SSL_LOGDIR} | sed 's#^\(.*[^/]\)$#\1/#'`
APACHE2_SCRIPT_ALIAS=`echo ${APACHE2_SCRIPT_ALIAS} | sed 's#^\(.*[^/]\)$#\1/#'`
APACHE2_SCRIPT_DIR=`echo ${APACHE2_SCRIPT_DIR} | sed 's#^\(.*[^/]\)$#\1/#'`


# -------------------------------------------------------------------------
# get crontab settings
# -------------------------------------------------------------------------
crontab_path="/var/cron/etc/root"
crontab_file="${crontab_path}/apache2"

# -------------------------------------------------------------------------
# get ip setting
# -------------------------------------------------------------------------
. /etc/config.d/base                 # include base config
IP_NET_1_STATIC_IP=$IP_ETH_1_IPADDR
# read dhcp leases
if [ "$IP_NET_1_STATIC_IP" = "no" ]
then
    leasefile=/var/lib/dhcp3/dhclient.eth0.leases
    if [ -f $leasefile ]
    then
        IP_NET_1_IPADDR=`grep fixed-address $leasefile | awk 'BEGIN { RS=""; FS="\n"} {print $NF}' | sed -e 's#[^0-9.]##g'`
    fi
else
    IP_NET_1_IPADDR=$IP_NET_1_STATIC_IP
fi

options="FollowSymLinks MultiViews"

if [ "$APACHE2_ENABLE_SSI" = "yes" ]
then
    options="$options Includes"
fi
if [ "$APACHE2_VIEW_DIRECTORY_CONTENT" = "yes" ]
then
    options="$options Indexes"
fi

if [ "$APACHE2_SSL" = "yes" ]
then
   if [ ! -f /usr/local/ssl/certs/ca.pem ]
   then
       myecho --info "Creating CA for SSL ..."
       /var/install/bin/certs-create-tls-certs ca batch
   fi
   if [ ! -f /usr/local/ssl/certs/apache.pem ]
   then
       myecho --info "Creating apache.pem"
       myecho --warn "Notice: The Common Name (you will type it in a moment) has to be the ServerName!"
       /var/install/bin/certs-create-tls-certs web batch alternate "apache" "$APACHE2_SERVER_NAME"
   fi
fi

#----------------------------------------------------------------------------------------
# activate modules
#----------------------------------------------------------------------------------------
myecho --info "activating modules..."
# auth_digest: delay apache start 60s
for module in alias auth_basic authn_file authz_groupfile authz_host authz_owner authz_user autoindex cern_meta cgi dbd deflate dir env headers log_config log_forensic mime mime_magic negotiation proxy_ajp proxy_balancer proxy proxy_connect proxy_ftp proxy_http rewrite setenvif userdir vhost_alias unixd mod_authz_host authz_core authn_core slotmem_shm
do
    [ -e /etc/apache2/mods-available/${module}.load ] && ln -sf /etc/apache2/mods-available/${module}.load /etc/apache2/mods-enabled/${module}.load
    [ -e /etc/apache2/mods-available/${module}.conf ] && ln -sf /etc/apache2/mods-available/${module}.conf /etc/apache2/mods-enabled/${module}.conf
done


#----------------------------------------------------------------------------------------
# activate diskcache
#----------------------------------------------------------------------------------------
modcache="$APACHE2_MOD_CACHE"
modfiles='cache
disk_cache'

if [ "$modcache" = "no" ]
then
    idx=0
    while [ "$idx" -le "${APACHE2_VHOST_N:-0}" ]
    do
        eval modcache='$APACHE2_VHOST_'$idx'_MOD_CACHE'
        [ "$modcache" = "yes" ] && break
        idx=`expr $idx + 1`
    done
fi
if [ "$modcache" = "yes" ]
then
    for module in $modfiles
    do
        [ -e /etc/apache2/mods-available/${module}.load ] && ln -sf /etc/apache2/mods-available/${module}.load /etc/apache2/mods-enabled/${module}.load
        [ -e /etc/apache2/mods-available/${module}.conf ] && ln -sf /etc/apache2/mods-available/${module}.conf /etc/apache2/mods-enabled/${module}.conf
    done
    create_crontab_file
else
    for module in $modfiles
    do
        rm -f /etc/apache2/mods-enabled/${module}.load
        rm -f /etc/apache2/mods-enabled/${module}.conf
    done
    rm -f ${crontab_file}
    # update crontab
    /var/install/config.d/cron > /dev/null 2>&1
fi

#----------------------------------------------------------------------------------------
# mod_compat
#----------------------------------------------------------------------------------------
if [ "$APACHE2_MOD_COMPAT" = "yes" ]
then
    ln -sf /etc/apache2/mods-available/access_compat.load /etc/apache2/mods-enabled/access_compat.load
else
    rm -f /etc/apache2/mods-enabled/access_compat.load
fi

#----------------------------------------------------------------------------------------
# APACHE2_HEADER_NOSNIFF
#----------------------------------------------------------------------------------------
rm -f /etc/apache2/mods-available/headers.conf
rm -f /etc/apache2/mods-enabled/headers.conf
if [ "$APACHE2_HEADER_NOSNIFF" = "yes" ]
then
    {
        echo "# X-Content-Type-Options"
        echo 'Header always set X-Content-Type-Options "nosniff"'
    } >> /etc/apache2/mods-available/headers.conf

    ln -sf /etc/apache2/mods-available/headers.conf /etc/apache2/mods-enabled/headers.conf
fi

#----------------------------------------------------------------------------------------
# APACHE2_HEADER_XSSPROTECTION
#----------------------------------------------------------------------------------------
if [ "$APACHE2_HEADER_XSSPROTECTION" = "yes" ]
then
    {
        echo "# X-Xss-Protection"
        echo 'Header always set X-Xss-Protection "1; mode=block"'
    } >> /etc/apache2/mods-available/headers.conf

    ln -sf /etc/apache2/mods-available/headers.conf /etc/apache2/mods-enabled/headers.conf
fi

#----------------------------------------------------------------------------------------
# APACHE2_HEADER_SAMEORIGIN
#----------------------------------------------------------------------------------------
if [ "$APACHE2_HEADER_SAMEORIGIN" = "yes" ]
then
    {
        echo "# X-Frame-Options"
        echo 'Header always set X-Frame-Options "SAMEORIGIN"'
    } >> /etc/apache2/mods-available/headers.conf

    ln -sf /etc/apache2/mods-available/headers.conf /etc/apache2/mods-enabled/headers.conf
fi

#----------------------------------------------------------------------------------------
# activate http2
#----------------------------------------------------------------------------------------
if [ "$APACHE2_ENABLE_HTTP2" = "yes" ]
then
    if [ "$APACHE2_MPM" != "prefork" ]
    then
        ln -sf /etc/apache2/mods-available/http2.load /etc/apache2/mods-enabled/http2.load
        ln -sf /etc/apache2/mods-available/http2.conf /etc/apache2/mods-enabled/http2.conf
    else
        myecho --warn "HTTP2 cannot be used if apache2 is using the worker prefork."
        myecho --warn "Setting APACHE2_ENABLE_HTTP2='no'"
        set_parameter_no "APACHE2_ENABLE_HTTP2"

    fi
else
    rm -f /etc/apache2/mods-enabled/http2.load
    rm -f /etc/apache2/mods-enabled/http2.conf
fi

#----------------------------------------------------------------------------------------
# activate ssl
#----------------------------------------------------------------------------------------
modfiles='ssl'
if [ "$APACHE2_SSL" = "yes" ]
then
    cat > /etc/apache2/mods-available/ssl.conf <<EOF
#==============================================================
# Do not edit this file, edit /etc/config.d/apache2
# Creation date: $EISDATE $EISTIME
AddType application/x-x509-ca-cert .crt
AddType application/x-pkcs7-crl    .crl

SSLProtocol             -ALL +TLSv1 -TLSv1 -TLSv1.1 +TLSv1.2 +TLSv1.3
SSLHonorCipherOrder     On
SSLCipherSuite          EECDH+ECDSA+AESGCM:EECDH+aRSA+AESGCM:EECDH+ECDSA+SHA384:EECDH+ECDSA+SHA256:EECDH+aRSA+SHA384:EECDH+aRSA+SHA256:EECDH+AESGCM:EECDH:EDH+AESGCM:EDH+aRSA:HIGH:!MEDIUM:!LOW:!aNULL:!eNULL:!LOW:!RC4:!MD5:!EXP:!PSK:!SRP:!DSS
SSLCompression          Off
SSLSessionCache         shmcb:/var/lib/apache/ssl_scache
SSLSessionCacheTimeout  300
SSLStrictSNIVHostCheck  off
SSLCACertificatePath    /usr/local/ssl/certs

EOF

    if [ "$APACHE2_SSL_OCSP" = "yes" ]
    then
        cat >> /etc/apache2/mods-available/ssl.conf <<EOF
SSLUseStapling          on
SSLStaplingResponderTimeout 5
SSLStaplingReturnResponderErrors off
SSLStaplingCache        shmcb:/run/ocsp(128000)
EOF
    fi

    for module in $modfiles
    do
        [ -e /etc/apache2/mods-available/${module}.load ] && ln -sf /etc/apache2/mods-available/${module}.load /etc/apache2/mods-enabled/${module}.load
        [ -e /etc/apache2/mods-available/${module}.conf ] && ln -sf /etc/apache2/mods-available/${module}.conf /etc/apache2/mods-enabled/${module}.conf
    done
else
    for module in $modfiles
    do
        rm -f /etc/apache2/mods-enabled/${module}.load
        rm -f /etc/apache2/mods-enabled/${module}.conf
    done
fi

#----------------------------------------------------------------------------------------
# activate webdav
#----------------------------------------------------------------------------------------
modfiles='dav
dav_fs'
webdav="no"
idx=1
while [ "$idx" -le "$APACHE2_DIR_N" ]
do
    eval webdav='$APACHE2_DIR_'$idx'_WEBDAV'
    [ "$webdav" = "yes" ] && break
    idx=`expr $idx + 1`
done
vidx=1
while [ "$vidx" -le "${APACHE2_VHOST_N:-0}" ]
do
    eval activevhost='$APACHE2_VHOST_'$vidx'_ACTIVE'
    if [ "$activevhost" = "yes" ]
    then
        idx=1
        eval tmpidx='$APACHE2_VHOST_'$vidx'_DIR_N'
        while [ "$idx" -le "$tmpidx" ]
        do
            eval webdavv='$APACHE2_VHOST_'$vidx'_DIR_'$idx'_WEBDAV'
            [ "$webdavv" = "yes" ] && break
            idx=`expr $idx + 1`
        done
    fi
    [ "$webdav" = "yes" ] && break
    vidx=`expr $vidx + 1`
done
if [ "$webdav" = "yes" -o "$webdavv" = "yes" ]
then
    for module in $modfiles
    do
        [ -e /etc/apache2/mods-available/${module}.load ] && ln -sf /etc/apache2/mods-available/${module}.load /etc/apache2/mods-enabled/${module}.load
        [ -e /etc/apache2/mods-available/${module}.conf ] && ln -sf /etc/apache2/mods-available/${module}.conf /etc/apache2/mods-enabled/${module}.conf
    done
#    create_crontab_file
else
    for module in $modfiles
    do
        rm -f /etc/apache2/mods-enabled/${module}.load
        rm -f /etc/apache2/mods-enabled/${module}.conf
    done
fi

myecho --info "done activating modules"

#----------------------------------------------------------------------------------------
# creating apache2.conf
#----------------------------------------------------------------------------------------
(
echo "#-------------------------------------------------------------------------------"
echo "# Apache configuration file generated by $pgmname"
echo "#"
echo "# Do not edit this file, edit $conffile or $pgmname"
echo "# Creation date: ${EISDATE} ${EISTIME}"
echo "#-------------------------------------------------------------------------------"
echo

if [ "$APACHE2_MPM" = "prefork" ]
then
    echo 'LoadModule mpm_prefork_module /usr/lib/apache2/modules/mod_mpm_prefork.so'
elif [ "$APACHE2_MPM" = "worker" ]
then
    echo 'LoadModule mpm_worker_module /usr/lib/apache2/modules/mod_mpm_worker.so'
elif [ "$APACHE2_MPM" = "event" ]
then
    echo 'LoadModule mpm_event_module /usr/lib/apache2/modules/mod_mpm_event.so'
fi

# check is apache2_phpX is installed and enabled, and give a warning if the mpm is not prefork
if [ "$APACHE2_MPM" != "prefork" ]
then
    php5_status=`eisman check apache2_php5`
    php7_status=`eisman check apache2_php7`
    php8_status=`eisman check apache2_php8`
    if [ "${php5_status}" != "not-installed" ]
    then
        if [ -f /etc/config.d/apache2_php5 ]
        then
            . /etc/config.d/apache2_php5
        fi
    fi
    if [ "${php7_status}" != "not-installed" ]
    then
        if [ -f /etc/config.d/apache2_php7 ]
        then
            . /etc/config.d/apache2_php7
        fi
    fi
    if [ "${php8_status}" != "not-installed" ]
    then
        if [ -f /etc/config.d/apache2_php8 ]
        then
            . /etc/config.d/apache2_php8
        fi
    fi
    if [ "${START_PHP5:-no}" = "yes" ] || [ "${START_PHP7:-no}" = "yes" ] || [ "${START_PHP8:-no}" = "yes" ]
    then
        mecho --warn "You have enabled the package apache2_php5/7/8, which is only working with the MPM prefork!"
    fi
fi

echo 'ServerRoot "/etc/apache2"'
echo 'PidFile /run/apache2.pid'

major=`uname -r | cut -f1 -d"."`
minor=`uname -r | cut -f2 -d"."`
if [  $major -lt 2  -o  $major -eq 2 -a $minor -lt 4  ]; then
  echo 'ScoreBoardFile /run/apache2_runtime_status'
fi

echo 'Timeout 300'
echo 'KeepAlive On'
echo "MaxKeepAliveRequests $APACHE2_MAX_KEEP_ALIVE_REQUESTS"
echo "KeepAliveTimeout $APACHE2_MAX_KEEP_ALIVE_TIMEOUT"
# calculate number of servers to start (1 per available core)
numberOfCores=`nproc`
echo "StartServers ${numberOfCores}"
echo "ServerLimit $APACHE2_MAX_CLIENTS"
if [ "${APACHE2_MPM}" != "prefork" ]
then
    echo "MinSpareThreads 25"
    echo "MaxSpareThreads 75"
    echo "ThreadLimit 64"
    echo "ThreadsPerChild 25"
fi
echo "MaxRequestWorkers $APACHE2_MAX_CLIENTS"
echo "MaxConnectionsPerChild $APACHE2_MAX_REQUESTS_PER_CHILD"
echo "# Include module configuration:"
echo "Include /etc/apache2/mods-enabled/*.load"
echo "Include /etc/apache2/mods-enabled/*.conf"
echo "User ${APACHE2_USERNAME}"
echo "Group ${APACHE2_GROUPNAME}"
echo "ServerAdmin $APACHE2_SERVER_ADMIN"
echo "ServerName $APACHE2_SERVER_NAME:$APACHE2_PORT"
echo "DocumentRoot \"$APACHE2_DOCUMENT_ROOT\""

if [ "${APACHE2_MPM}" != "prefork" ]
then
     idx=1
     while [ ${idx} -le ${APACHE2_VHOST_N:-0} ]
     do
         eval active='${APACHE2_VHOST_'${idx}'_ACTIVE}'

         if [ "${active}" != "no" ]
         then
             php_fpm_default=false
             break
         fi
         idx=$((${idx} + 1 ))
     done

     if "${php_fpm_default:-true}"
     then
         echo "Include /etc/apache2/php-pools/php8-fpm-default.conf"
    fi
fi

host_alias=$( find ${vhost_alias_path} \
              -maxdepth 1 \
              -type f \
              -name "host-${APACHE2_SERVER_NAME}-*" \
              -printf '%f\n' )

for v in ${host_alias}
do
    echo "$(cat ${vhost_alias_path}/${v})"
done

echo "EnableSendfile Off"
if [ "$APACHE2_MOD_CACHE" = "yes" ]
then
    echo "CacheEnable mem /"
    echo "<IfModule mod_cache_disk.c>"
    echo "    CacheEnable disk /"
    echo "</IfModule>"
fi

uses_vhost="no"
uses_vhost_atall="no"    # marker to see if there is any activated vhost
nameIpMixture="no"
hasAsterisk="no"
hasIp="no"
idx=1
while [ "$idx" -le "${APACHE2_VHOST_N:-0}" ]
do
    eval active='$APACHE2_VHOST_'$idx'_ACTIVE'
    eval ip='$APACHE2_VHOST_'$idx'_IP'
    eval port='$APACHE2_VHOST_'$idx'_PORT'
    eval servername='$APACHE2_VHOST_'$idx'_SERVER_NAME'
    eval serveralias='$APACHE2_VHOST_'$idx'_SERVER_ALIAS'
    eval mail='$APACHE2_VHOST_'$idx'_SERVER_ADMIN'
    eval docroot='$APACHE2_VHOST_'$idx'_DOCUMENT_ROOT'
    eval scriptalias='$APACHE2_VHOST_'$idx'_SCRIPT_ALIAS'
    eval scriptdir='$APACHE2_VHOST_'$idx'_SCRIPT_DIR'
    eval errorlog='$APACHE2_VHOST_'$idx'_ERROR_LOG'
    eval accesslog='$APACHE2_VHOST_'$idx'_ACCESS_LOG'
    eval accesscontrol='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL'
    eval ssl='$APACHE2_VHOST_'$idx'_SSL'
    eval sslport='$APACHE2_VHOST_'$idx'_SSL_PORT'
    eval forcessl='$APACHE2_VHOST_'$idx'_SSL_FORCE'
    eval sslcertname='$APACHE2_VHOST_'$idx'_SSL_CERT_NAME'

    if [ "$active" = "yes" ]
    then
        uses_vhost="yes"
        uses_vhost_atall="yes"
    else
        idx=`expr $idx + 1`
        continue
    fi
    if [ "$ip" = "*" ]
    then
        hasAsterisk="yes"
        ip="0.0.0.0"
    else
        hasIp="yes"
    fi

    ports="$port "
    if [ "$ssl" = "yes" -a "$APACHE2_SSL" = "yes" ]
    then
        if [ ! "x$sslport" = "x" ]
        then
            ports="$port $sslport"
        fi
    fi
    for single_port in $ports
    do
        if [ ! "`echo \"$ipports\" | grep \"$ip:$single_port\"`" ]
        then
            ipports="$ipports $ip:$single_port "
        fi
    done
    idx=`expr $idx + 1`
done

# check whether there is a mixture of name- and ip-based vhosts
if [ "$hasAsterisk" = "yes" ]
then
    if [ "$hasIp" = "yes" ]
    then
        nameIpMixture="yes"
        if [ ! "`echo \"$ipports\" | grep \"${IP_NET_1_IPADDR}:${APACHE2_PORT}\"`" ]
        then
            ipports="$ipports ${IP_NET_1_IPADDR}:${APACHE2_PORT} "
        fi
        if [ "$APACHE2_SSL" = "yes" ]
        then
            if [ ! "`echo \"$ipports\" | grep \"${IP_NET_1_IPADDR}:${APACHE2_SSL_PORT}\"`" ]
            then
                ipports="$ipports ${IP_NET_1_IPADDR}:${APACHE2_SSL_PORT} "
            fi
        fi
    fi
else
    if [ ! "`echo \"$ipports\" | grep \"${IP_NET_1_IPADDR}:${APACHE2_PORT}\"`" ]
    then
        ipports="$ipports ${IP_NET_1_IPADDR}:${APACHE2_PORT} "
    fi
    if [ "$APACHE2_SSL" = "yes" ]
    then
        if [ ! "`echo \"$ipports\" | grep \"${IP_NET_1_IPADDR}:${APACHE2_SSL_PORT}\"`" ]
        then
            ipports="$ipports ${IP_NET_1_IPADDR}:${APACHE2_SSL_PORT} "
        fi
    fi
fi

if [ "$uses_vhost_atall" != "no" ]
then
    if [ "$nameIpMixture" = "no" ]
    then
        iApacheSslSet=0
        for ipport in $ipports
        do
            echo "Listen $ipport"
            if [ ! $ipport = "0.0.0.0:$APACHE2_SSL_PORT" ]
            then
                iApacheSslSet=1
            fi
        done
        if [ "$APACHE2_SSL" = "yes" -a $iApacheSslSet = 0 ]
        then
            echo "Listen 0.0.0.0:$APACHE2_SSL_PORT"
            iApacheSslSet=1
        fi
    else
        for ipport in $ipports
        do
            if [ $ipport != "0.0.0.0:$APACHE2_PORT" ] &&  [ $ipport != "0.0.0.0:$APACHE2_SSL_PORT" ]
            then
                echo "Listen $ipport"
            fi
        done
    fi
else
    echo "Listen 0.0.0.0:$APACHE2_PORT"
    if [ "$APACHE2_SSL" = "yes" ]
    then
        echo "Listen 0.0.0.0:$APACHE2_SSL_PORT"
    fi
fi

echo '<Directory />'
echo '    Options FollowSymLinks'
echo '    AllowOverride None'
echo '</Directory>'

echo "<Directory $APACHE2_DOCUMENT_ROOT>"
echo "    Options $options"
echo '    AllowOverride All'
if [ "$APACHE2_ACCESS_CONTROL_TYPE_ALL" = "yes" ]
then
    echo "    Require all granted"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE_IP" = "yes" ]
then
    for ipaddr in ${APACHE2_DIR_1_ACCESS_CONTROL_IP}
    do
        echo "    Require ip ${ipaddr}"
    done
elif [ "$APACHE2_ACCESS_CONTROL_TYPE_HOST" = "yes" ]
then
    echo "    Require host $APACHE2_DIR_1_ACCESS_CONTROL_HOST"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE_LOCAL" = "yes" ]
then
    echo "    Require local"
fi
echo '</Directory>'

echo "<Directory /home/*/public_html>"
echo "    Options $options"
echo '    AllowOverride All'
if [ "$APACHE2_ACCESS_CONTROL_TYPE_ALL" = "yes" ]
then
    echo "    Require all granted"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE_IP" = "yes" ]
then
    for ipaddr in ${APACHE2_DIR_1_ACCESS_CONTROL_IP}
    do
        echo "    Require ip ${ipaddr}"
    done
elif [ "$APACHE2_ACCESS_CONTROL_TYPE_HOST" = "yes" ]
then
    echo "    Require host $APACHE2_DIR_1_ACCESS_CONTROL_HOST"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE_LOCAL" = "yes" ]
then
    echo "    Require local"
fi
echo '</Directory>'

idx=1
while [ "$idx" -le "$APACHE2_DIR_N" ]
do
    eval active='$APACHE2_DIR_'$idx'_ACTIVE'
    eval useAlias='$APACHE2_DIR_'$idx'_ALIAS'
    eval alias='$APACHE2_DIR_'$idx'_ALIAS_NAME'
    eval path='$APACHE2_DIR_'$idx'_PATH'
    eval auth_name='$APACHE2_DIR_'$idx'_AUTH_NAME'
    eval auth_type='$APACHE2_DIR_'$idx'_AUTH_TYPE'
    eval auth_n='$APACHE2_DIR_'$idx'_AUTH_N'
    eval cgi='$APACHE2_DIR_'$idx'_CGI'
    eval ssi='$APACHE2_DIR_'$idx'_SSI'
    eval access_type_all='$APACHE2_DIR_'$idx'_ACCESS_CONTROL_TYPE_ALL'
    eval access_type_ip='$APACHE2_DIR_'$idx'_ACCESS_CONTROL_TYPE_IP'
    eval access_ip='$APACHE2_DIR_'$idx'_ACCESS_CONTROL_IP'
    eval access_type_host='$APACHE2_DIR_'$idx'_ACCESS_CONTROL_TYPE_HOST'
    eval access_host='$APACHE2_DIR_'$idx'_ACCESS_CONTROL_HOST'
    eval access_type_local='$APACHE2_DIR_'$idx'_ACCESS_CONTROL_TYPE_LOCAL'
    eval content='$APACHE2_DIR_'$idx'_VIEW_DIR_CONTENT'
    eval webdav='$APACHE2_DIR_'$idx'_WEBDAV'

    if [ "$active" = "yes" ]
    then
        if [ "$useAlias" = "yes" ]
        then
            echo "Alias $alias $path"
        fi

        #echo "Adding directory $path ..." >`tty`
        echo "<Directory $path>"
        echo -n '    Options FollowSymLinks MultiViews'
        if [ "$ssi" = "yes" ]
        then
            echo -n ' Includes'
        fi
        if [ "$cgi" != "none" ]
        then
            echo -n ' ExecCGI'
        fi

        if [ "$content" = "yes" ]
        then
            echo ' Indexes'
        else
            echo
        fi
        if [ "$cgi" != "none" ]
        then
            echo '    AddHandler cgi-script '$cgi
        fi
        if [ "$ssi" = "yes" ]
        then
            echo '    AddType text/html .shtml'
            echo '    AddHandler server-parsed .shtml'
        fi
        if [ "$auth_n" != "0" ]
        then
            if [ ! -d /usr/share/apache2/passwd ]
            then
                mkdir /usr/share/apache2/passwd
            fi
            if [ "${auth_type}" = "Basic" ]
            then
                echo '    AuthType Basic'
                echo '    AuthBasicProvider file'
            else
                echo '    <IfModule !mod_auth_digest.c>'
                echo '        LoadModule auth_digest_module /usr/lib/apache2/modules/mod_auth_digest.so'
                echo '    </IfModule>'
                echo '    AuthType Digest'
                echo '    AuthDigestDomain "'$auth_name'"'
                echo '    AuthDigestProvider file'
            fi
            echo '    AuthName "'$auth_name'"'
            echo '    AuthUserFile /usr/share/apache2/passwd/passwords.'$idx
            echo '    require valid-user'
            rm -f /usr/share/apache2/passwd/passwords.$idx
            touch /usr/share/apache2/passwd/passwords.$idx

            idx2=1
            while [ "$idx2" -le "$auth_n" ]
            do
                eval user='$APACHE2_DIR_'$idx'_AUTH_'$idx2'_USER'
                eval pass='$APACHE2_DIR_'$idx'_AUTH_'$idx2'_PASS'

                if [ "${auth_type}" = "Basic" ]
                then
                    /usr/bin/htpasswd -b /usr/share/apache2/passwd/passwords.$idx $user $pass 2>/dev/null
                else
                    # hash the username, realm, and password
                    htdigest_hash=`printf "$user:$auth_name:$pass" | md5sum -`
                    # build an htdigest appropriate line, and tack it onto the file
                    echo "${user}:${auth_name}:${htdigest_hash:0:32}" >> /usr/share/apache2/passwd/passwords.$idx
                fi
                idx2=`expr $idx2 + 1`
            done
            chown -R ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} /usr/share/apache2/passwd
            chmod 700 /usr/share/apache2/passwd
            chmod 600 /usr/share/apache2/passwd/*
        else
            if [ "$access_type_all" = "yes" ]
            then
                echo "    Require all granted"
            elif [ "$access_type_ip" = "yes" ]
            then
                echo "    Require ip $access_ip"
            elif [ "$access_type_host" = "yes" ]
            then
                echo "    Require host $access_host"
            elif [ "$access_type_local" = "yes" ]
            then
                echo "    Require local"
            fi
        fi

        if [ "$webdav" = "yes" ]
        then
            mkdir -p /usr/share/apache2/var
            chown ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} /usr/share/apache2/var

            echo "    Dav on"
        fi

        echo '    AllowOverride All'
        echo '</Directory>'

        if [ ! -d $path ]
        then
            mkdir -p $path
            touch $path/index.html
            echo "<h1>GEHEIM!</h1>">$path/index.html
            chown -R ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} $path
        fi
    fi
    idx=`expr $idx + 1`
done

echo "UserDir public_html"
echo "DirectoryIndex $APACHE2_DIRECTORY_INDEX"
echo 'AccessFileName .htaccess'

echo '<Files ~ "^\.ht">'
echo '    Require all denied'
echo '</Files>'
echo ''

echo 'UseCanonicalName Off'
echo 'TypesConfig /etc/apache2/mime.types'
echo 'MIMEMagicFile /etc/apache2/magic'

if [ "$APACHE2_HOSTNAME_LOOKUPS" = "yes" ]
then
    echo 'HostnameLookups On'
else
    echo 'HostnameLookups Off'
fi

echo "ErrorLog $APACHE2_ERROR_LOG"
echo "LogLevel $APACHE2_LOG_LEVEL"
echo 'LogFormat "%h %l %u %t \"%r\" %>s %b \"%{Referer}i\" \"%{User-Agent}i\" %I %O" combined'
echo 'SetEnvIf Remote_Addr "127\.0\.0\.1" dontlog'
echo "CustomLog $APACHE2_ACCESS_LOG combined env=!dontlog"
echo 'ServerTokens Minor'
echo "ServerSignature $APACHE2_SERVER_SIGNATURE"

echo "ScriptAlias $APACHE2_SCRIPT_ALIAS $APACHE2_SCRIPT_DIR"
echo "<Directory $APACHE2_SCRIPT_DIR>"
echo '    AllowOverride All'
echo '    Options None'
if [ "$APACHE2_ACCESS_CONTROL_TYPE" = "all" ]
then
    echo "    Require all granted"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE" = "ip" ]
then
    echo "    Require ip $APACHE2_ACCESS_CONTROL"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE" = "host" ]
then
    echo "    Require host $APACHE2_ACCESS_CONTROL"
elif [ "$APACHE2_ACCESS_CONTROL_TYPE" = "local" ]
then
    echo "    Require local"
fi
echo '</Directory>'

echo "Include /etc/apache2/AddIcons.inc"

echo "Include /etc/apache2/AddLanguages.inc"

if [ "$APACHE2_ENABLE_SSI" = "yes" ]
then
    echo 'AddType text/html .shtml'
    echo 'AddHandler server-parsed .shtml'
fi

idx=1
while [ "$idx" -le "$APACHE2_ERROR_DOCUMENT_N" ]
do
    eval error='$APACHE2_ERROR_DOCUMENT_'$idx'_ERROR'
    eval doc='$APACHE2_ERROR_DOCUMENT_'$idx'_DOCUMENT'

    echo "ErrorDocument $error $doc"
    idx=`expr $idx + 1`
done

echo 'BrowserMatch "Mozilla/2" nokeepalive'
echo 'BrowserMatch "MSIE 4\.0b2;" nokeepalive downgrade-1.0 force-response-1.0'
echo 'BrowserMatch "RealPlayer 4\.0" force-response-1.0'
echo 'BrowserMatch "Java/1\.0" force-response-1.0'
echo 'BrowserMatch "JDK/1\.0" force-response-1.0'
echo 'BrowserMatch "Microsoft Data Access Internet Publishing Provider" redirect-carefully'
echo 'BrowserMatch "MS FrontPage" redirect-carefully'
echo 'BrowserMatch "^WebDrive" redirect-carefully'
echo 'BrowserMatch "^WebDAVFS/1.[012]" redirect-carefully'
echo 'BrowserMatch "^gnome-vfs/1.0" redirect-carefully'
echo 'BrowserMatch "^XML Spy" redirect-carefully'
echo 'BrowserMatch "^Dreamweaver-WebDAV-SCM1" redirect-carefully'

if [ "$APACHE2_SSL" = "yes" ]
then
    if [ $APACHE2_VHOST_N -eq 0 -o "$uses_vhost_atall" = "no" ]
    then
        echo "<VirtualHost _default_:$APACHE2_SSL_PORT>"
        echo "    ServerName $APACHE2_SERVER_NAME:$APACHE2_SSL_PORT"
        echo "    ErrorLog ${APACHE2_ERROR_LOG}.ssl"
        echo "    CustomLog ${APACHE2_ACCESS_LOG}.ssl combined"
        echo "    <Directory $APACHE2_DOCUMENT_ROOT>"
        echo "        Options $options"
        echo '        AllowOverride All'
        if [ "$APACHE2_ACCESS_CONTROL_TYPE" = "all" ]
        then
            echo "    Require all granted"
        elif [ "$APACHE2_ACCESS_CONTROL_TYPE" = "ip" ]
        then
            echo "    Require ip $APACHE2_ACCESS_CONTROL"
        elif [ "$APACHE2_ACCESS_CONTROL_TYPE" = "host" ]
        then
            echo "    Require host $APACHE2_ACCESS_CONTROL"
        elif [ "$APACHE2_ACCESS_CONTROL_TYPE" = "local" ]
        then
            echo "    Require local"
        fi
        echo '    </Directory>'
        echo "    SSLEngine On"
        echo "    SSLCertificateFile /usr/local/ssl/certs/apache.pem"
        echo "    SSLCertificateKeyFile /usr/local/ssl/certs/apache.pem"
        echo '    <Files ~ \"\.(pl|cgi|shtml|phtml|php?)$\">'
        echo "        SSLOptions +StdEnvVars"
        echo "    </Files>"
        echo "<Directory \"$APACHE2_SCRIPT_DIR\">"
        echo "    SSLOptions +StdEnvVars"
        echo "</Directory>"
        echo '    SetEnvIf User-Agent ".*MSIE.*" nokeepalive ssl-unclean-shutdown downgrade-1.0 force-response-1.0'
        echo '    CustomLog '$APACHE2_SSL_LOGDIR'ssl_request_log "%t %h %{SSL_PROTOCOL}x %{SSL_CIPHER}x \"%r\" %b"'
        echo "</VirtualHost>"
    fi
fi

if [ "$APACHE2_VHOST_N" != "0" ]
then
    idx=1
    anyVHostActive="no"
    anyVHostSSLActive="no"
    while [ "$idx" -le "${APACHE2_VHOST_N:-0}" ]
    do
        eval active='$APACHE2_VHOST_'$idx'_ACTIVE'
        eval ssl='$APACHE2_VHOST_'$idx'_SSL'
        if [ "$active" = "yes" ]
        then
            anyVHostActive="yes"
        fi
        if [ "$ssl" = "yes" ]
        then
            anyVHostSSLActive="yes"
        fi
        idx=`expr $idx + 1`
    done
    #if [ "$anyVHostActive" = "yes" ]
    #then
    #    for ipport in $ipports
    #    do
    #        echo "NameVirtualHost $ipport"
    #    done
    #fi
fi

# create logrotate file before insert vhost logfile entries
cat > ${testroot}/etc/logrotate.d/apache2 <<EOF
#---------------------------------------------------------------------
# /etc/logrotate.d/apache2 file generated by /var/install/config.d/apache2.sh
#
# Do not edit this file, edit /etc/config.d/apache2.sh
# Creation Date: $EISDATE $EISTIME
#---------------------------------------------------------------------
EOF


idx=1
while [ "$idx" -le "${APACHE2_VHOST_N:-0}" ]
do
    eval active='$APACHE2_VHOST_'$idx'_ACTIVE'
    eval ip='$APACHE2_VHOST_'$idx'_IP'
    eval port='$APACHE2_VHOST_'$idx'_PORT'
    eval servername='$APACHE2_VHOST_'$idx'_SERVER_NAME'
    eval serveralias='$APACHE2_VHOST_'$idx'_SERVER_ALIAS'
    eval mail='$APACHE2_VHOST_'$idx'_SERVER_ADMIN'
    eval docroot='$APACHE2_VHOST_'$idx'_DOCUMENT_ROOT'
    eval scriptalias='$APACHE2_VHOST_'$idx'_SCRIPT_ALIAS'
    eval scriptdir='$APACHE2_VHOST_'$idx'_SCRIPT_DIR'
    eval errorlog='$APACHE2_VHOST_'$idx'_ERROR_LOG'
    eval accesslog='$APACHE2_VHOST_'$idx'_ACCESS_LOG'
    eval access_type_all='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL_TYPE_ALL'
    eval access_type_ip='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL_TYPE_IP'
    eval access_ip='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL_IP'
    eval access_type_host='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL_TYPE_HOST'
    eval access_host='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL_HOST'
    eval access_type_local='$APACHE2_VHOST_'$idx'_ACCESS_CONTROL_TYPE_LOCAL'
    eval content='$APACHE2_VHOST_'$idx'_VIEW_DIRECTORY_CONTENT'
    eval ssi='$APACHE2_VHOST_'$idx'_ENABLE_SSI'
    eval modcache='$APACHE2_VHOST_'$idx'_MOD_CACHE'
    eval ssl='$APACHE2_VHOST_'$idx'_SSL'
    eval sslport='$APACHE2_VHOST_'$idx'_SSL_PORT'
    eval usests='$APACHE2_VHOST_'$idx'_SSL_USESTS'
    eval usests_subdomains='$APACHE2_VHOST_'$idx'_SSL_USESTS_SUBDOMAINS'
    eval forcessl='$APACHE2_VHOST_'$idx'_SSL_FORCE'
    eval sslcertname='$APACHE2_VHOST_'$idx'_SSL_CERT_NAME'

    # append vhost logfiles
    if [ "$active" = "yes" ]
    then
        if ! grep -q "$accesslog"  ${testroot}/etc/logrotate.d/apache2
        then
            echo -n "$accesslog " >> ${testroot}/etc/logrotate.d/apache2
        fi
        if ! grep -q "$errorlog"  ${testroot}/etc/logrotate.d/apache2
        then
            echo -n "$errorlog " >> ${testroot}/etc/logrotate.d/apache2
        fi
        if [ "$ssl" = "yes" ]
        then
            if ! grep -q "${accesslog}.ssl"  ${testroot}/etc/logrotate.d/apache2
            then
                echo -n "${accesslog}.ssl " >> ${testroot}/etc/logrotate.d/apache2
            fi
            if ! grep -q "${errorlog}.ssl"  ${testroot}/etc/logrotate.d/apache2
            then
                echo -n "${errorlog}.ssl " >> ${testroot}/etc/logrotate.d/apache2
            fi
        fi
    fi

    options="FollowSymLinks MultiViews"

    if [ "$ssi" = "yes" ]
    then
        options="$options Includes"
    fi
    if [ "$content" = "yes" ]
    then
        options="$options Indexes"
    fi

    if [ "$active" != "yes" ]
    then
        idx=`expr $idx + 1`
        continue
    fi

    if [ "$ssl" = "yes" -a "$APACHE2_SSL" != "yes" ]
    then
        {
        myecho --info "You enabled SSL for the VHost no. $idx ($servername)"
        myecho --info "but this will not take affect because APACHE2_SSL is not set to 'yes'"
        }>`tty`
    fi

    if [ ! -d $docroot ]
    then
        mkdir -p $docroot

        {
        echo "<html>"
        echo "<body>"
        echo "<h1>Der VirtualHost <em>$servername</em> wurde erfolgreich eingerichtet!</h1>"
        echo "HTML-Dateien muessen nach <em>$docroot</em> geladen werden, CGI-Scripts nach <i>$scriptdir</i>.<br>"
        echo "Die Access-Logfile ist <em>$accesslog</em><br>"
        echo "Die Error-Logfile ist <em>$errorlog</em><p>"
        echo "Zugriff auf diesen VirtualHost hat <em>$accesscontrol</em>"
        echo "<h1>The VirtualHost <em>$servername</em> was created succesfully!</h1>"
        echo "HTML files must be loaded into <em>$docroot</em>, the CGI-Scripts into <em>$scriptdir</em>.<br>"
        echo "The access logfile is <em>$accesslog</em><br>"
        echo "The error logfile is <em>$errorlog</em><p>"
        echo "Access to this VirtualHost has <em>$accesscontrol</em>"
        echo "</body>"
        echo "</html>"
        }>$docroot/index.html

       chown ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} -R $docroot
    fi

    if [ ! -d $scriptdir ]
    then
        mkdir -p $scriptdir
        chown ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} $scriptdir
    fi

    accesslog_dir=`dirname $accesslog`
    if [ ! -d $accesslog_dir ]
    then
        mkdir -p $accesslog_dir
        chown ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} $accesslog_dir
    fi

    errorlog_dir=`dirname $errorlog`
    if [ ! -d $errorlog_dir ]
    then
        mkdir -p $error_log_dir
        chown ${APACHE2_USERNAME}:${APACHE2_GROUPNAME} $errorlog_dir
    fi

    echo "<VirtualHost $ip:$port>"
    echo "    ServerName $servername:$port"

    if [ "x$serveralias" != "x" ]
    then
        echo "    ServerAlias $serveralias"
    fi

    echo "    ServerAdmin $mail"
    echo "    DocumentRoot $docroot"

    if [ "${APACHE2_MPM}" != "prefork" ]
    then
        echo "    Include /etc/apache2/php-pools/php8-fpm-default.conf"
    fi

    vhost_alias=$( find ${vhost_alias_path} \
                   -maxdepth 1 \
                   -type f \
                   -name "vhost-${servername}-*" \
                   -printf '%f\n' )

    for v in ${vhost_alias}
    do
        echo "    $(cat ${vhost_alias_path}/${v})"
    done

    echo "    ScriptAlias $scriptalias $scriptdir"

    if [ "$modcache" = "yes" ]
    then
        echo "    CacheEnable mem /"
        echo "    <IfModule mod_cache_disk.c>"
        echo "        CacheEnable disk /"
        echo "    </IfModule>"
    else
        [ "$APACHE2_MOD_CACHE" = "yes" ] && echo "    CacheDisable /"
    fi

    echo "    <Directory $scriptdir>"
    echo '        AllowOverride All'
    echo '        Options None'
    if [ "$access_type_all" = "yes" ]
    then
        echo "    Require all granted"
    elif [ "$access_type_ip" = "yes" ]
    then
        echo "    Require ip $access_ip"
    elif [ "$access_type_host" = "yes" ]
    then
        echo "    Require host $access_host"
    elif [ "$access_type_local" = "yes" ]
    then
        echo "    Require local"
    fi
    echo '    </Directory>'

    echo "    <Directory $docroot>"
    echo "        AllowOverride All"
    echo "        Options $options"
    if [ "$access_type_all" = "yes" ]
    then
        echo "    Require all granted"
    elif [ "$access_type_ip" = "yes" ]
    then
        echo "    Require ip $access_ip"
    elif [ "$access_type_host" = "yes" ]
    then
        echo "    Require host $access_host"
    elif [ "$access_type_local" = "yes" ]
    then
        echo "    Require local"
    fi
    echo "    </Directory>"

    if [ "$APACHE2_SSL" = "yes" -a "$ssl" = "yes" -a "$forcessl" = "yes" ]
    then
        echo "    Redirect permanent / https://$servername:$sslport/"
    fi

    echo "    ErrorLog $errorlog"
    echo "    CustomLog $accesslog combined"

    ###########
    createVHostDirDirective
    ###########

    echo "</VirtualHost>"

    ### SSL VIRTUALHOST
    if [ "$sslcertname" = "" ]
    then
      sslcertname="apache"
    fi

    if [ "$APACHE2_SSL" = "yes" -a "$ssl" = "yes" ]
    then
        echo "<VirtualHost "$ip":"$sslport">"
        echo "    SSLEngine On"
        echo "    SSLCertificateFile /usr/local/ssl/certs/$sslcertname.pem"
        echo "    SSLCertificateKeyFile /usr/local/ssl/certs/$sslcertname.pem"
        if [ "$usests" = "yes" ]
        then
            if [ "$usests_subdomains" = "yes" ]
            then
                echo 'Header add Strict-Transport-Security "max-age=15768000; includeSubDomains"'
            else
                echo 'Header add Strict-Transport-Security "max-age=15768000"'
            fi
        fi
        echo '    <Files ~ "\.(pl|cgi|shtml|phtml|php|php?)$">'
        echo "        SSLOptions +StdEnvVars"
        echo "    </Files>"

        echo "    <Directory $docroot>"
        echo "        AllowOverride All"
        echo "        Options $options"
        if [ "$access_type_all" = "yes" ]
        then
            echo "    Require all granted"
        elif [ "$access_type_ip" = "yes" ]
        then
            echo "    Require ip $access_ip"
        elif [ "$access_type_host" = "yes" ]
        then
            echo "    Require host $access_host"
        elif [ "$access_type_local" = "yes" ]
        then
            echo "    Require local"
        fi
        echo "    </Directory>"

        echo "    <Directory \"$scriptdir\">"
        echo "        SSLOptions +StdEnvVars"
        echo "    </Directory>"
        echo '    SetEnvIf User-Agent ".*MSIE.*" nokeepalive ssl-unclean-shutdown downgrade-1.0 force-response-1.0'
        echo "    ServerName $servername:$sslport"

        if [ "$serveralias" != "" ]
        then
            echo "    ServerAlias $serveralias"
        fi

        echo "    ServerAdmin $mail"
        echo "    DocumentRoot $docroot"

        if [ "${APACHE2_MPM}" != "prefork" ]
        then
            echo "    Include /etc/apache2/php-pools/php8-fpm-default.conf"
        fi

        vhost_alias=$( find ${vhost_alias_path} \
                       -maxdepth 1 \
                       -type f \
                       -name "vhost-${servername}-*" \
                       -printf '%f\n' )

        for v in ${vhost_alias}
        do
            echo "    $(cat ${vhost_alias_path}/${v})"
        done

        echo "    ScriptAlias $scriptalias $scriptdir"
        echo "    ErrorLog ${errorlog}.ssl"
        echo "    CustomLog ${accesslog}.ssl combined"
        ###########
        createVHostDirDirective
        ###########
        echo "</VirtualHost>"

        (
        if [ ! -f /usr/local/ssl/certs/${sslcertname}.pem ]
        then
            myecho --warn "The certificate $sslcertname doesn't exist"
            if /var/install/bin/ask "Do you want to create it now"
            then
                myecho --info "Creating $sslcertname.pem"
                myecho --warn "Notice: The Common Name (you will type it in a moment) has to be the ServerName!"

                /var/install/bin/certs-create-tls-certs web batch alternate "$sslcertname" "$servername"
            fi
        fi
        )>`tty`
    fi

    idx=`expr $idx + 1`
done

#----------------------------------------------------------------------------------------
# check whether ssl and a combination of name and ip based hosts is active
#----------------------------------------------------------------------------------------
if [ "$nameIpMixture" = "yes" -a "$APACHE2_SSL" = "yes" ]
then
    {
        myecho
        myecho --warn "You enabled SSL but not all of your VHOSTs have a separate IP adress!"
        myecho --info "Therefore SSL will not work correctly because of the nature of the SSL protocol."
        myecho
     }>`tty`
fi


echo "Include /etc/apache2/AddParams.conf"

# fix for dual php4/5 system
if [ -f /tmp/php_set ] ; then
   rm -f /tmp/php_set
fi

# This will include the OUTPUT of all files in /var/install/config.d/apache2.conf.*.sh into
# apache2.conf

for i in `ls /var/install/config.d/apache2.conf.*.sh 2> /dev/null`; do
    sh $i
done

)>/etc/apache2/apache2.conf

#----------------------------------------------------------------------------------------
# append logrotate body
#----------------------------------------------------------------------------------------
standard=''
if ! grep -q "${APACHE2_ACCESS_LOG}"  ${testroot}/etc/logrotate.d/apache2
then
    standard="${APACHE2_ACCESS_LOG}"
fi
if ! grep -q "${APACHE2_ERROR_LOG}"  ${testroot}/etc/logrotate.d/apache2
then
    standard="${standard} ${APACHE2_ERROR_LOG}"
fi
if [ "$APACHE2_SSL" = "yes" ]
then
    if ! grep -q "${APACHE2_ACCESS_LOG}.ssl"  ${testroot}/etc/logrotate.d/apache2
    then
        standard="${standard} ${APACHE2_ACCESS_LOG}.ssl"
    fi
    if ! grep -q "${APACHE2_ERROR_LOG}.ssl"  ${testroot}/etc/logrotate.d/apache2
    then
        standard="${standard} ${APACHE2_ERROR_LOG}.ssl"
    fi
    standard="${standard} ${APACHE2_SSL_LOGDIR}ssl_request_log"
fi
cat >> ${testroot}/etc/logrotate.d/apache2 <<EOF
${standard} {
    $APACHE2_LOG_INTERVAL
    missingok
    rotate $APACHE2_LOG_COUNT
    compress
    delaycompress
    notifempty
    create 640 root ${APACHE2_GROUPNAME}
    sharedscripts
    postrotate
        /usr/sbin/service --quiet stop apache2.service
        sleep 3
        /usr/sbin/service --quiet start apache2.service
    endscript
}
EOF

# -------------------------------------------------------------------------
# check for packages
# -------------------------------------------------------------------------
mods_plugin_path='/etc/apache2/mods-plugins'
mods_plugin=$( find ${mods_plugin_path} \
               -maxdepth 1 \
               -type f \
               -name '*' \
               -printf '%f\n' 2>/dev/null)

for p in ${mods_plugin}
do
    ${mods_plugin_path}/${p} ${quiet}
done

packages_to_check="mod_python
                   mod_jk
                   apache2_php5
                   apache2_php7
                   apache2_php8"

packagesOK="yes"
for p in $packages_to_check
do
    if [ -f /var/install/packages/$p ]
    then
        if ! echo ${mods_plugin} | grep -q "${p}"
        then
            if grep -q check_httpd /var/install/config.d/${p}.sh
            then
                /var/install/config.d/${p}.sh check_httpd
                if [ $? = 0 ]
                then
                    packagesOK="yes"
                fi
            fi
        fi
    fi
done

# check if DOCUMENT ROOT exists
if [ ! -d $APACHE2_DOCUMENT_ROOT ]
then
    if /var/install/bin/ask "The directory $APACHE2_DOCUMENT_ROOT does not exist. Do you want to create it?"
    then
        myecho --info "Creating directory $APACHE2_DOCUMENT_ROOT"
        mkdir -p $APACHE2_DOCUMENT_ROOT
    else
        myecho --warn "The directory $APACHE2_DOCUMENT_ROOT does not exist"
        packagesOK="no"
    fi
fi

#
if [ "$packagesOK" = "no" ]
then
    myecho --warn "Setting START_APACHE2='no'"
    # 1 rewrite Parameter from 'yes' to 'no'
    set_parameter_no "START_APACHE2"
    anykey
fi

check_for_autostart